@@ -24,6 +24,12 @@ FILE *out_fd;
 
 #define H_SIZE (64 * 1024)
 
+#ifdef __clang__
+#define NO_SANITIZE_ALIGNMENT __attribute__((no_sanitize("alignment")))
+#else
+#define NO_SANITIZE_ALIGNMENT
+#endif
+
 void *ag_malloc(size_t size);
 void *ag_realloc(void *ptr, size_t size);
 void *ag_calloc(size_t nelem, size_t elsize);
@@ -47,8 +53,6 @@ typedef struct {
 
 ag_stats stats;
 
-typedef const char *(*strncmp_fp)(const char *, const char *, const size_t, const size_t, const size_t[], const size_t *);
-
 /* Union to translate between chars and words without violating strict aliasing */
 typedef union {
     char as_chars[sizeof(uint16_t)];
@@ -67,13 +71,9 @@ void generate_hash(const char *find, const size_t f_len, uint8_t *H, const int c
 size_t ag_max(size_t a, size_t b);
 
 const char *boyer_moore_strnstr(const char *s, const char *find, const size_t s_len, const size_t f_len,
-                                const size_t alpha_skip_lookup[], const size_t *find_skip_lookup);
-const char *boyer_moore_strncasestr(const char *s, const char *find, const size_t s_len, const size_t f_len,
-                                    const size_t alpha_skip_lookup[], const size_t *find_skip_lookup);
+                                const size_t alpha_skip_lookup[], const size_t *find_skip_lookup, const int case_insensitive);
 const char *hash_strnstr(const char *s, const char *find, const size_t s_len, const size_t f_len, uint8_t *h_table, const int case_sensitive);
 
-strncmp_fp get_strstr(enum case_behavior opts);
-
 size_t invert_matches(const char *buf, const size_t buf_len, match_t matches[], size_t matches_len);
 void realloc_matches(match_t **matches, size_t *matches_size, size_t matches_len);
 void compile_study(pcre **re, pcre_extra **re_extra, char *q, const int pcre_opts, const int study_opts);
